@@ -1,6 +1,6 @@
 package Auto::Mata;
 # ABSTRACT: A simple, reliable finite state machine
-$Auto::Mata::VERSION = '0.06';
+$Auto::Mata::VERSION = '0.07';
 
 use strict;
 use warnings;
@@ -22,6 +22,7 @@ our @EXPORT = qw(
   to
   on
   with
+  using
 );
 
 our $DEBUG = $ENV{DEBUG_AUTOMATA};
@@ -31,13 +32,16 @@ my $State = declare 'State', as Tuple[$Ident, Any];
 my $Type  = declare 'Type',  as InstanceOf['Type::Tiny'];
 coerce $Type, from Undef, via { Any };
 
-my $Transition = declare 'Transition', as Dict[initial => $Type, transform => Maybe[CodeRef]];
-my $Transform  = declare 'Transform',  as Dict[initial => $Type, transform => CodeRef];
+my $Transition = declare 'Transition', as Dict[
+  to        => $Ident,
+  initial   => $Type,
+  transform => Maybe[CodeRef],
+];
 
 my $Automata = declare 'Automata', as Dict[
   ready => Maybe[$Ident],
   term  => Maybe[$Ident],
-  map   => Map[$Ident, Map[$Ident, $Transition]],
+  map   => Map[$Ident, ArrayRef[$Transition]],
 ];
 
 
@@ -47,8 +51,7 @@ sub machine (&) {
   #-----------------------------------------------------------------------------
   # Define the machine parameters
   #-----------------------------------------------------------------------------
-  my %map;
-  my %fsm = (ready => undef, term => undef, map => \%map);
+  my %fsm = (ready => undef, term => undef, map => {});
 
   do {
     local $_ = \%fsm;
@@ -56,12 +59,16 @@ sub machine (&) {
     validate();
   };
 
-  my $terminal = declare $fsm{term}, as Tuple[Enum[$fsm{term}], Any];
-  my %state    = ($fsm{term} => $terminal);
+  my %map      = %{$fsm{map}};
+  my $ready    = $fsm{ready};
+  my $term     = $fsm{term};
+  my $Terminal = declare 'Terminal', as Tuple[Enum[$term], Any];
+
+  my %final = ($term => $Terminal);
 
   foreach my $from (keys %map) {
-    my @next = map { $map{$from}{$_}{initial} } keys %{$map{$from}};
-    $state{$from} = declare $from, as reduce { $a | $b } @next;
+    my @next = map { $_->{initial} } @{$map{$from}};
+    $final{$from} = declare $from, as reduce { $a | $b } @next;
   }
 
   #-----------------------------------------------------------------------------
@@ -74,14 +81,15 @@ sub machine (&) {
     # Use this to build a matching function that calls the appropriate mutator
     # for that transisiton.
     #---------------------------------------------------------------------------
-    foreach my $to (keys %{$map{$from}}) {
-      my $final   = $state{$to};
-      my $initial = $map{$from}{$to}{initial};
-      my $with    = $map{$from}{$to}{transform};
+    foreach my $transition (@{$map{$from}}) {
+      my $to      = $transition->{to};
+      my $initial = $transition->{initial};
+      my $with    = $transition->{transform};
+      my $final   = $final{$to};
 
       push @match, $initial, sub {
         my ($from, $input) = @$_;
-        debug('%s -> %s', $from, $to);
+        debug('%s -> %s: %s', $from, $to, explain($input));
 
         do { local $_ = $input; $input = $with->() } if $with;
         my $state = [$to, $input];
@@ -109,15 +117,27 @@ sub machine (&) {
   # Return function that builds a transition engine for the given input
   #-----------------------------------------------------------------------------
   return sub {
-    my $state = $fsm{ready};
+    my $interactive = shift;
+
+    my $state = $ready;
     my $done;
 
-    sub {
+    my $iter = sub (\$) {
       return if $done;
       ($state, $_[0]) = $transform->([$state, $_[0]]);
-      $done = $state eq $fsm{term};
+      $done = $Terminal->check([$state, $_[0]]);
       wantarray ? ($state, $_[0]) : $state;
     };
+
+    return $iter if $interactive;
+
+    return sub (\$) {
+      while (my ($state, $acc) = $iter->($_[0])) {
+        ;
+      }
+
+      return $_[0];
+    };
   };
 }
 
@@ -128,6 +148,7 @@ sub term     ($)   { goto \&terminal }
 sub to       ($;%) { (to   => shift, @_) }
 sub on       ($;%) { (on   => shift, @_) }
 sub with     (&;%) { (with => shift, @_) }
+sub using    (&;%) { goto \&with }
 
 
 my $_transition_args;
@@ -139,17 +160,25 @@ sub transition ($%) {
   my ($arg, %param) = @_;
   my ($from, $to, $on, $with) = $_transition_args->($arg, @param{qw(to on with)});
 
-  croak "transition from state $from to $to is already defined"
-    if exists $_->{map}{$from}{$to};
+  $_->{map}{$from} ||= [];
 
-  my $init = declare "${from}_TO_${to}_INITIAL", as Tuple[Enum[$from], $on];
+  my $name = $on->name;
+  my $init = declare "${from}_to_${to}_on_${name}", as Tuple[Enum[$from], $on];
+  debug("New state: $init");
 
-  $_->{map}{$from} ||= {};
+  foreach my $next (@{$_->{map}{$from}}) {
+    croak "identical transition $next->{initial} already defined"
+      if $init == $next->{initial};
+  }
 
-  $_->{map}{$from}{$to} = {
+  my $transition = {
+    to        => $to,
     initial   => $init,
     transform => $with,
   };
+
+  # Add this contraint to the list of matches
+  push @{$_->{map}{$from}}, $transition;
 }
 
 #-------------------------------------------------------------------------------
@@ -205,7 +234,8 @@ sub validate {
     unless keys %{$_->{map}};
 
   croak 'no transition defined for ready state'
-    unless $_->{map}{$_->{ready}};
+    unless $_->{map}{$_->{ready}}
+        && @{$_->{map}{$_->{ready}}};
 
   my $is_terminated;
 
@@ -213,7 +243,9 @@ sub validate {
     croak 'invalid transition from terminal state detected'
       if $from eq $_->{term};
 
-    foreach my $to (keys %{$_->{map}{$from}}) {
+    foreach my $next (@{$_->{map}{$from}}) {
+      my $to = $next->{to};
+
       if ($to eq $_->{term}) {
         $is_terminated = 1;
         next;
@@ -243,7 +275,7 @@ Auto::Mata - A simple, reliable finite state machine
 
 =head1 VERSION
 
-version 0.06
+version 0.07
 
 =head1 SYNOPSIS
 
@@ -267,7 +299,7 @@ version 0.06
 
   my $fsm = machine {
     ready 'READY';
-    terminal 'TERM';
+    term  'TERM';
 
     transition 'READY', to 'FIRST',
       on $NoData,
@@ -286,11 +318,7 @@ version 0.06
   };
 
   my $prog = $fsm->();
-  my $data;
-
-  while (my ($state, $output) = $prog->($data)) {
-    ;
-  }
+  my $data = $prog->();
 
   printf "Hello %s %s, aged %d years!\n", @$data;
 
@@ -330,29 +358,41 @@ C<Auto::Mata> is an C<Exporter>. All subroutines are exported by default.
 Creates a lexical context in which a state machine is defined. Returns a
 function that creates new instances of the defined automata.
 
-The automata instance is itself a function that performs a single transition
-per call. It accepts a single reference value as input representing the
-program's current state. This value in conjunction with the current state label
-is matched (see L</on>) against the transition table (defined with
+The automata instance is itself a builder function. When called, it returns a
+new function that accepts the initial program state as input and returns the
+final state.
+
+This instance function's input value in conjunction with the current state
+label is matched (see L</on>) against the transition table (defined with
 L</transition>) to determine the next state.
 
 Once a match has been made, the action defined for the transition (using
 L</with>) will be executed. During evaluation of the L</with> function, C<$_>
 is a reference to the input value.
 
+Note that the input state is modified in place during the transition.
+
+  # Define the state machine
+  my $builder = machine { ... };
+
+  # Create an instance
+  my $instance = $builder->();
+
+  # Run the program to get the result
+  my $result = $instance->($state_data);
+
+If the builder function is called with a true value as the first argument, it
+will instead build an interactive iterator that performs a single transition
+per call. It accepts a single value as input representing the program's current
+state.
+
 The return value is new state's label in scalar context, the label and state in
 list context, and C<undef> after the terminal state has been reached.
 
-Note that the reference used as input may (and likely will have) have been
-modified during the transition. It will always be the reference passed as
-input.
-
   # Define the state machine
-  my $builder = machine {
-    ...
-  };
+  my $builder = machine { ... };
 
-  # Create an instance of the machine that operates on $state.
+  # Create an instance of the machine
   my $program = $builder->();
 
   # Run the program
@@ -391,13 +431,6 @@ L</with> is permitted to the current input. If L</with> is not specified, the
 input remains unchanged. Once the transition is complete, the symbol supplied
 by L</to> will identify the current program state.
 
-It is an error to have two identical transitions, even with different
-constraints. This is intentional. A transition that matches two different
-states is, in fact, two distinct transitions, and the program should be modeled
-as such in order to prevent errors due to unexpected or improperly checked
-data. In general, it is a good idea to be as specific as possible with the type
-constraints used to define the initial transition state.
-
 The first transition is always from the "ready" state. The final transition is
 always to the "terminal" state. There may be no transitions from the "terminal"
 state.
@@ -421,6 +454,14 @@ A code block whose return value is the mutable state used to determine the next
 transition to perform. Within the code block C<$_> is a reference to the
 program state.
 
+=item using
+
+Alias for L</with> that may be used to avoid conflicts with other packages
+exporting C<with>. Prevent the export of C<with> in the typical way (see
+L<Exporter/Specialised-Import-Lists>).
+
+  use Auto::Mata '!with';
+
 =back
 
   machine {
